AgritourismPackMaster = {}; 

function AgritourismPackMaster:DistanceFrom(x1,y1,x2,y2) return math.sqrt((x2 - x1) ^ 2 + (y2 - y1) ^ 2) end

function AgritourismPackMaster:GetMaxGuestByHouse(plc1)
    local Price = plc1.storeItem.price;
    if Price < 5000 then Price = 5000 end;
    local Returning = math.floor(Price / 5000);
    return Returning;
end;

function AgritourismPackMaster:GetAllManagementBuildings()
    ManagementBuildings = {}
    for i=#g_currentMission.placeableSystem.placeables, 1, -1 do
        local Cplaceable = g_currentMission.placeableSystem.placeables[i];
        if Cplaceable ~= nil and Cplaceable.storeItem.name == g_i18n:getText("shopItem_ManagementBuilding") then  ManagementBuildings[table.getn(ManagementBuildings) + 1] = Cplaceable; end;
    end
    return ManagementBuildings;
end;

function AgritourismPackMaster:GetAllHousingBuildings()
    HousingBuildings = {}
    for i=#g_currentMission.placeableSystem.placeables, 1, -1 do
        local Cplaceable = g_currentMission.placeableSystem.placeables[i];
        if Cplaceable ~= nil and Cplaceable.spec_farmhouse then  HousingBuildings[table.getn(HousingBuildings) + 1] = Cplaceable; end;
    end
    return HousingBuildings;
end;

function AgritourismPackMaster:IsInEffectiveRange(plc1,plc2)
    return AgritourismPackMaster:DistanceFrom(plc1.position.x,plc1.position.z,plc2.position.x,plc2.position.z) < 100;
end;

function AgritourismPackMaster:IsSpecializationInRange(plc1,SpecName)
    for i=#g_currentMission.placeableSystem.placeables, 1, -1 do
        local Cplaceable = g_currentMission.placeableSystem.placeables[i];
        if Cplaceable ~= nil and AgritourismPackMaster:IsInEffectiveRange(plc1,Cplaceable) and AgritourismPackMaster:IsInSameFarmland(plc1,Cplaceable)then
            for id, spec in pairs(Cplaceable.specializations) do
                local name = Cplaceable.specializationNames[id]
                if name == SpecName then return true; end;
            end
        end;
    end

    return false;
end;

function AgritourismPackMaster:HasSpecialization(plc1,SpecName)
    for id, spec in pairs(plc1.specializations) do
        local name = plc1.specializationNames[id]
        if name == SpecName then return true; end;
    end
    return false;
end;

function AgritourismPackMaster:GetNumOfDecorationsInRange(plc1)
    local Number = 0;
    for i=#g_currentMission.placeableSystem.placeables, 1, -1 do
        local Cplaceable = g_currentMission.placeableSystem.placeables[i];
        if Cplaceable ~= nil then
            if AgritourismPackMaster:IsInEffectiveRange(plc1,Cplaceable) and AgritourismPackMaster:IsInSameFarmland(plc1,Cplaceable) then
                
                if AgritourismPackMaster:HasSpecialization(Cplaceable,"greenhouse") == false and AgritourismPackMaster:HasSpecialization(Cplaceable,"husbandry") == false and 
                AgritourismPackMaster:HasSpecialization(Cplaceable,"beehive") == false and AgritourismPackMaster:HasSpecialization(Cplaceable,"weatherStation") == false and 
                AgritourismPackMaster:HasSpecialization(Cplaceable,"chargingStation") == false and AgritourismPackMaster:HasSpecialization(Cplaceable,"solarPanels") == false and 
                AgritourismPackMaster:HasSpecialization(Cplaceable,"windTurbine") == false and  AgritourismPackMaster:HasSpecialization(Cplaceable,"farmhouse") == false and 
                AgritourismPackMaster:HasSpecialization(Cplaceable,"productionPoint") == false and  AgritourismPackMaster:HasSpecialization(Cplaceable,"sellingStation") == false and 
                AgritourismPackMaster:HasSpecialization(Cplaceable,"silo") == false and  AgritourismPackMaster:HasSpecialization(Cplaceable,"greenhouse") == false and 
                AgritourismPackMaster:HasSpecialization(Cplaceable,"greenhouse") == false and  AgritourismPackMaster:HasSpecialization(Cplaceable,"greenhouse") == false 
                then Number = Number +1; end;
            end;
        end;
    end
    return Number;
end;

function AgritourismPackMaster:IsInSameFarmland(plc1,plc2)
    return plc1.farmlandId == plc2.farmlandId
end;

function AgritourismPackMaster:UpdateHousingQuantity() 
    local ManagementBuildings = AgritourismPackMaster:GetAllManagementBuildings();
    local HousingBuildings = AgritourismPackMaster:GetAllHousingBuildings();

    if ManagementBuildings == nil or table.getn(ManagementBuildings) < 1 then return; end;
    
    for i = 1, table.getn(ManagementBuildings) do
        local CurrentManagementBuilding = ManagementBuildings[i];
        local NumberOfBuildings = 1;
        local GuestsCapacity = 30;
        for z = 1, table.getn(HousingBuildings) do
            local CurrentHousingBuilding = HousingBuildings[z];
            if NumberOfBuildings < 26 and AgritourismPackMaster:IsInEffectiveRange(CurrentManagementBuilding,CurrentHousingBuilding) and AgritourismPackMaster:IsInSameFarmland(ManagementBuildings,HousingBuildings) then
                NumberOfBuildings = NumberOfBuildings + 1;
                GuestsCapacity = GuestsCapacity + AgritourismPackMaster:GetMaxGuestByHouse(CurrentHousingBuilding);
            end;
        end

        CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:setFillLevel(NumberOfBuildings, FillType.HOUSING);
        CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:setFillLevel(GuestsCapacity, FillType.GUESTQTY);
    end
end;

function AgritourismPackMaster:UpdateFoodQuality() 
    local ManagementBuildings = AgritourismPackMaster:GetAllManagementBuildings();

    if ManagementBuildings == nil or table.getn(ManagementBuildings) < 1 then return; end;
    
    for i = 1, table.getn(ManagementBuildings) do
        local FoodsNumber = 0;
        local CurrentManagementBuilding = ManagementBuildings[i];

        if CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.GRAPEJUICE) > 0 then FoodsNumber = FoodsNumber + 1 end;
        if CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.BREAD) > 0 then FoodsNumber = FoodsNumber + 1 end;
        if CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.BUTTER) > 0 then FoodsNumber = FoodsNumber + 1 end;
        if CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.LETTUCE) > 0 then FoodsNumber = FoodsNumber + 1 end;
        if CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.OLIVE) > 0 then FoodsNumber = FoodsNumber + 1 end;
        if CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.TOMATO) > 0 then FoodsNumber = FoodsNumber + 1 end;
        if CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.EGG) > 0 then FoodsNumber = FoodsNumber + 1 end;
        if CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.CHEESE) > 0 then FoodsNumber = FoodsNumber + 1 end;
        if CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.MILK) > 0 then FoodsNumber = FoodsNumber + 1 end;
        if CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.CEREAL) > 0 then FoodsNumber = FoodsNumber + 1 end;
        if CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.HONEY) > 0 then FoodsNumber = FoodsNumber + 1 end;
        if CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.STRAWBERRY) > 0 then FoodsNumber = FoodsNumber + 1 end;
        if CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.CHOCOLATE) > 0 then FoodsNumber = FoodsNumber + 1 end;

        CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:setFillLevel(math.floor((FoodsNumber/13)*100), FillType.FOODQUALITY);
    end
end;

function AgritourismPackMaster:UpdateAttractiveness() 
    local ManagementBuildings = AgritourismPackMaster:GetAllManagementBuildings();

    if ManagementBuildings == nil or table.getn(ManagementBuildings) < 1 then return; end;
    
    for i = 1, table.getn(ManagementBuildings) do
        local CurrentManagementBuilding = ManagementBuildings[i];
        local Attractiveness = 0;

        if AgritourismPackMaster:IsSpecializationInRange(CurrentManagementBuilding,"greenhouse") == true then Attractiveness = Attractiveness+10 end;
        if AgritourismPackMaster:IsSpecializationInRange(CurrentManagementBuilding,"husbandry") == true then Attractiveness = Attractiveness+10 end;
        if AgritourismPackMaster:IsSpecializationInRange(CurrentManagementBuilding,"beehive") == true then Attractiveness = Attractiveness+10 end;

        if AgritourismPackMaster:IsSpecializationInRange(CurrentManagementBuilding,"weatherStation") == true then Attractiveness = Attractiveness+10 end;
        if AgritourismPackMaster:IsSpecializationInRange(CurrentManagementBuilding,"chargingStation") == true then Attractiveness = Attractiveness+10 end;
        if AgritourismPackMaster:IsSpecializationInRange(CurrentManagementBuilding,"solarPanels") == true or AgritourismPackMaster:IsSpecializationInRange(CurrentManagementBuilding,"windTurbine") == true then Attractiveness = Attractiveness+10 end;

        Attractiveness = Attractiveness + AgritourismPackMaster:GetNumOfDecorationsInRange(CurrentManagementBuilding);

        if Attractiveness > 100 then Attractiveness = 100; end;
        CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:setFillLevel(Attractiveness, FillType.ATTRACTIONS);
    end
end;

function AgritourismPackMaster:UpdateOverallRating() 
    local ManagementBuildings = AgritourismPackMaster:GetAllManagementBuildings();

    if ManagementBuildings == nil or table.getn(ManagementBuildings) < 1 then return; end;
    
    for i = 1, table.getn(ManagementBuildings) do
        local CurrentManagementBuilding = ManagementBuildings[i];
        local Rating = CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.FOODQUALITY)/2;
        Rating = Rating + CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.ATTRACTIONS)/2;
        Rating = math.floor(Rating);
        CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:setFillLevel(Rating, FillType.RATING);
    end
end;

function AgritourismPackMaster:IsResortOpertional(plc1)
    if table.getn(plc1.spec_productionPoint.productionPoint.activeProductions) < 1 then return false; end;
    if plc1.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.WATER) < 1 then return false end;
    
    if plc1.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.GRAPEJUICE) < 1 and 
    plc1.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.BREAD) < 1 and 
    plc1.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.BUTTER) < 1 and 
    plc1.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.LETTUCE) < 1 and 
    plc1.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.OLIVE) < 1 and 
    plc1.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.TOMATO) < 1 and 
    plc1.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.EGG) < 1 and 
    plc1.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.CHEESE) < 1 and 
    plc1.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.MILK) < 1 and 
    plc1.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.CEREAL) < 1 and 
    plc1.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.HONEY) < 1 and 
    plc1.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.STRAWBERRY) < 1 and 
    plc1.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.CHOCOLATE) < 1 
    then return false end;

    return true;
end;

function AgritourismPackMaster:DailyRevenue()
    local ManagementBuildings = AgritourismPackMaster:GetAllManagementBuildings();
    if ManagementBuildings == nil or table.getn(ManagementBuildings) < 1 then return; end;
    
    for i = 1, table.getn(ManagementBuildings) do
        local CurrentManagementBuilding = ManagementBuildings[i];
        local GuestsNumber = CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.GUESTQTY);
        local Rating = CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.RATING);
        Rating = Rating / 15;

        
        if Rating < 1 then Rating = 1 end;
        local Consumed = 0;

        if AgritourismPackMaster:IsResortOpertional(CurrentManagementBuilding) == true then
            for z=1,GuestsNumber do

                local Level = 0;

                Level = CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.WATER);
                if Level > 5 then
                    CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:setFillLevel(Level-5, FillType.WATER);
                end;

                Level = CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.GRAPEJUICE);
                if Level > 0 then
                    CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:setFillLevel(Level-1, FillType.GRAPEJUICE);
                    Consumed = Consumed + 1*g_fillTypeManager:getFillTypeByIndex(FillType.GRAPEJUICE).pricePerLiter;
                end;

                Level = CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.BREAD);
                if Level > 0 then
                    CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:setFillLevel(Level-1, FillType.BREAD);
                    Consumed = Consumed + 1*g_fillTypeManager:getFillTypeByIndex(FillType.BREAD).pricePerLiter;
                end;

                Level = CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.BUTTER);
                if Level > 0 then
                    CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:setFillLevel(Level-1, FillType.BUTTER);
                    Consumed = Consumed + 1*g_fillTypeManager:getFillTypeByIndex(FillType.BUTTER).pricePerLiter;
                end;

                Level = CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.LETTUCE);
                if Level > 0 then
                    CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:setFillLevel(Level-1, FillType.LETTUCE);
                    Consumed = Consumed + 1*g_fillTypeManager:getFillTypeByIndex(FillType.LETTUCE).pricePerLiter;
                end;

                Level = CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.OLIVE);
                if Level > 0 then
                    CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:setFillLevel(Level-1, FillType.OLIVE);
                    Consumed = Consumed + 1*g_fillTypeManager:getFillTypeByIndex(FillType.OLIVE).pricePerLiter;
                end;

                Level = CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.TOMATO);
                if Level > 0 then
                    CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:setFillLevel(Level-1, FillType.TOMATO);
                    Consumed = Consumed + 1*g_fillTypeManager:getFillTypeByIndex(FillType.TOMATO).pricePerLiter;
                end;

                Level = CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.EGG);
                if Level > 0 then
                    CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:setFillLevel(Level-1, FillType.EGG);
                    Consumed = Consumed + 1*g_fillTypeManager:getFillTypeByIndex(FillType.EGG).pricePerLiter;
                end;

                Level = CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.CHEESE);
                if Level > 0 then
                    CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:setFillLevel(Level-1, FillType.CHEESE);
                    Consumed = Consumed + 1*g_fillTypeManager:getFillTypeByIndex(FillType.CHEESE).pricePerLiter;
                end;

                Level = CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.MILK);
                if Level > 0 then
                    CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:setFillLevel(Level-1, FillType.MILK);
                    Consumed = Consumed + 1*g_fillTypeManager:getFillTypeByIndex(FillType.MILK).pricePerLiter;
                end;

                Level = CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.CEREAL);
                if Level > 0 then
                    CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:setFillLevel(Level-1, FillType.CEREAL);
                    Consumed = Consumed + 1*g_fillTypeManager:getFillTypeByIndex(FillType.CEREAL).pricePerLiter;
                end;

                Level = CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.HONEY);
                if Level > 0 then
                    CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:setFillLevel(Level-1, FillType.HONEY);
                    Consumed = Consumed + 1*g_fillTypeManager:getFillTypeByIndex(FillType.HONEY).pricePerLiter;
                end;

                Level = CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.STRAWBERRY);
                if Level > 0 then
                    CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:setFillLevel(Level-1, FillType.STRAWBERRY);
                    Consumed = Consumed + 1*g_fillTypeManager:getFillTypeByIndex(FillType.STRAWBERRY).pricePerLiter;
                end;

                Level = CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.CHOCOLATE);
                if Level > 0 then
                    CurrentManagementBuilding.spec_productionPoint.productionPoint.storage:setFillLevel(Level-1, FillType.CHOCOLATE);
                    Consumed = Consumed + 1*g_fillTypeManager:getFillTypeByIndex(FillType.CHOCOLATE).pricePerLiter;
                end;
            end

            g_currentMission:addMoney(Consumed*Rating, CurrentManagementBuilding:getOwnerFarmId(), MoneyType.PROPERTY_INCOME, true);
            --print(Consumed*Rating);
            --g_currentMission:showMoneyChange(MoneyType.PROPERTY_INCOME, nil, false, CurrentManagementBuilding:getOwnerFarmId());
        end;
    end
end;

function AgritourismPackMaster:loadMap()	
    if g_currentMission:getIsServer() or g_currentMission.isMasterUser then
        g_messageCenter:subscribe(MessageType.MINUTE_CHANGED, self.MinuteChanged, self);
        g_messageCenter:subscribe(MessageType.DAY_CHANGED, self.DayChanged, self)
    end;
end;

function AgritourismPackMaster:MinuteChanged()
    AgritourismPackMaster:UpdateHousingQuantity();
    AgritourismPackMaster:UpdateFoodQuality();
    AgritourismPackMaster:UpdateAttractiveness();
    AgritourismPackMaster:UpdateOverallRating();
end

function AgritourismPackMaster:DayChanged()
    AgritourismPackMaster:DailyRevenue();
end

addModEventListener(AgritourismPackMaster);