PlaceablePlacementGui = {}; 

local SelectedPlaceable = nil;
local InfoTextMessage = nil;
local TextInfoWarning = false;
local TextInfoHint = false;

function PlaceablePlacementGui:DistanceFrom(x1,y1,x2,y2) return math.sqrt((x2 - x1) ^ 2 + (y2 - y1) ^ 2) end

function PlaceablePlacementGui:IsInAnotherManagementBuildingArea(x,z)
    for i=#g_currentMission.placeableSystem.placeables, 1, -1 do
        local Iplaceable = g_currentMission.placeableSystem.placeables[i];
        if Iplaceable.storeItem.name == g_i18n:getText("shopItem_ManagementBuilding") then
            if PlaceablePlacementGui:DistanceFrom(x,z,Iplaceable.position.x,Iplaceable.position.z) < 100*2 then return true end;
        end;
    end

    return false;
end;

function PlaceablePlacementGui:GetManagementBuildingInArea(x,z)
    for i=#g_currentMission.placeableSystem.placeables, 1, -1 do
        local Iplaceable = g_currentMission.placeableSystem.placeables[i];
        if Iplaceable.storeItem.name == g_i18n:getText("shopItem_ManagementBuilding") then
            if PlaceablePlacementGui:DistanceFrom(x,z,Iplaceable.position.x,Iplaceable.position.z) < 100 then return Iplaceable; end;
        end;
    end

    return nil;
end;

function PlaceablePlacementGui:HasSpecialization(plc1,SpecName)
    for id, spec in pairs(plc1.specializations) do
        local name = plc1.specializationNames[id]
        if name == SpecName then return true; end;
    end
    return false;
end;

function PlaceablePlacementGui:IsAviableForResort(Cplaceable)
    if PlaceablePlacementGui:HasSpecialization(Cplaceable,"greenhouse") == false and PlaceablePlacementGui:HasSpecialization(Cplaceable,"husbandry") == false and 
    PlaceablePlacementGui:HasSpecialization(Cplaceable,"beehive") == false and PlaceablePlacementGui:HasSpecialization(Cplaceable,"weatherStation") == false and 
    PlaceablePlacementGui:HasSpecialization(Cplaceable,"chargingStation") == false and PlaceablePlacementGui:HasSpecialization(Cplaceable,"solarPanels") == false and 
    PlaceablePlacementGui:HasSpecialization(Cplaceable,"windTurbine") == false and  PlaceablePlacementGui:HasSpecialization(Cplaceable,"farmhouse") == false and 
    PlaceablePlacementGui:HasSpecialization(Cplaceable,"productionPoint") == false and  PlaceablePlacementGui:HasSpecialization(Cplaceable,"sellingStation") == false and 
    PlaceablePlacementGui:HasSpecialization(Cplaceable,"silo") == false and  PlaceablePlacementGui:HasSpecialization(Cplaceable,"greenhouse") == false and 
    PlaceablePlacementGui:HasSpecialization(Cplaceable,"greenhouse") == false and  PlaceablePlacementGui:HasSpecialization(Cplaceable,"greenhouse") == false 
    then return true end;
    
    if PlaceablePlacementGui:HasSpecialization(Cplaceable,"greenhouse") == true then return true; end;
    if PlaceablePlacementGui:HasSpecialization(Cplaceable,"husbandry") == true then return true; end;
    if PlaceablePlacementGui:HasSpecialization(Cplaceable,"beehive") == true then return true; end;

    if PlaceablePlacementGui:HasSpecialization(Cplaceable,"weatherStation") == true then return true; end;
    if PlaceablePlacementGui:HasSpecialization(Cplaceable,"chargingStation") == true then return true; end;
    if PlaceablePlacementGui:HasSpecialization(Cplaceable,"solarPanels") == true or PlaceablePlacementGui:HasSpecialization(Cplaceable,"windTurbine") == true then return true; end;

    return false;
end;

function PlaceablePlacementGui:GetMaxGuestByPrice(Price)
    if Price < 5000 then Price = 5000 end;
    local Returning = math.floor(Price / 5000);
    return Returning;
end;

function PlaceablePlacementGui:IsInEffectiveRange(plc1,plc2)
    return PlaceablePlacementGui:DistanceFrom(plc1.position.x,plc1.position.z,plc2.position.x,plc2.position.z) < 100;
end;

function PlaceablePlacementGui:getHasOverlapWithZones(superFunc, zones, x, y, z, rotY)
    InfoTextMessage = nil;
    TextInfoWarning = false;
    TextInfoHint = false;

    if SelectedPlaceable ~= nil then
        if SelectedPlaceable.storeItem.name == g_i18n:getText("shopItem_ManagementBuilding") then 
            if PlaceablePlacementGui:IsInAnotherManagementBuildingArea(x,z) then TextInfoWarning = true; InfoTextMessage = g_i18n:getText("text_ManagementBuildingDistance"); return true; end;
        end;

        for id, spec in pairs(SelectedPlaceable.specializations) do
            local name = self.specializationNames[id]
            --print(name);
        end

        if SelectedPlaceable.spec_farmhouse then 
            local ManagementBuildingInArea = PlaceablePlacementGui:GetManagementBuildingInArea(x,z);
            if ManagementBuildingInArea ~= nil then
                InfoTextMessage = g_i18n:getText("text_HausingBuldingFor").." '"..ManagementBuildingInArea:getName().."'".."\n";  
                InfoTextMessage = InfoTextMessage .. g_i18n:getText("text_HausingHandlePre").." "..PlaceablePlacementGui:GetMaxGuestByPrice(SelectedPlaceable.storeItem.price).." "..g_i18n:getText("text_HausingHandlePost");
            end;
            if ManagementBuildingInArea == nil then
                TextInfoHint = true;
                InfoTextMessage = g_i18n:getText("text_HausingAvaible").."\n";
                InfoTextMessage = InfoTextMessage .. g_i18n:getText("text_HausingHandlePre").." "..PlaceablePlacementGui:GetMaxGuestByPrice(SelectedPlaceable.storeItem.price).." "..g_i18n:getText("text_HausingHandlePost");
            end;
        end;

        if PlaceablePlacementGui:IsAviableForResort(SelectedPlaceable) then
            local ManagementBuildingInArea = PlaceablePlacementGui:GetManagementBuildingInArea(x,z);
            if ManagementBuildingInArea ~= nil then
                InfoTextMessage = g_i18n:getText("text_BuildingConnected").." '"..ManagementBuildingInArea:getName().."'";
            
            end;
            if ManagementBuildingInArea == nil then
                TextInfoHint = true;
                InfoTextMessage = g_i18n:getText("text_BuildingAvaible");  
            end;
        end;
    end;

    return superFunc(self,zones,x,y,z,rotY);
end;

function PlaceablePlacementGui:SelectedPlaceableChange(superFunc, zones, x, y, z, rotY)
    SelectedPlaceable = self.placeable;
end;

function PlaceablePlacementGui:update()
    if g_currentMission:getIsClient() and SelectedPlaceable ~= nil and InfoTextMessage ~= nil then
        setTextColor(0.1,1,0.1,1);

        if TextInfoWarning == true then setTextColor(1,0.1,0.1,1); end;
        if TextInfoHint == true then setTextColor(0.1,0.1,1,1); end;

        local fontSize = g_gameSettings:getValue("uiScale") * 0.020;
        local posX = 0.5;
        local posY = 0.95;
    
        local width = getTextWidth(fontSize, InfoTextMessage);
        posX = posX - width/2
    
        renderText(posX, posY, fontSize, InfoTextMessage);
    end;
end

function PlaceablePlacementGui:onDelete()
    if g_currentMission:getIsClient() then
    SelectedPlaceable = nil;
    InfoTextMessage = nil;
    end;
end

function PlaceablePlacementGui:onPreFinalizePlacement()
    if g_currentMission:getIsClient() and SelectedPlaceable~=nil and SelectedPlaceable.storeItem.name == g_i18n:getText("shopItem_ManagementBuilding") then
        g_gui:showTextInputDialog({
            text = g_i18n:getText("text_Rename"),
            confirmText = g_i18n:getText("text_Confirm"),
            defaultText = "New resort",
            callback = function(text,yes)
                self:setName(text, nil);
            end
        });
    end;
end

ConstructionBrushPlaceable.loadedPlaceable = Utils.appendedFunction(ConstructionBrushPlaceable.loadedPlaceable, PlaceablePlacementGui.SelectedPlaceableChange);
PlaceablePlacement.getHasOverlapWithZones = Utils.overwrittenFunction(PlaceablePlacement.getHasOverlapWithZones, PlaceablePlacementGui.getHasOverlapWithZones);
PlaceablePlacement.onPreFinalizePlacement = Utils.appendedFunction(PlaceablePlacement.onPreFinalizePlacement, PlaceablePlacementGui.onPreFinalizePlacement);
PlaceablePlacement.onDelete = Utils.appendedFunction(PlaceablePlacement.onDelete, PlaceablePlacementGui.onDelete);
addModEventListener(PlaceablePlacementGui);